/***********************************************************
 * $Id$
 *
 * ATOS client - ATOS client modules
 *
 * Copyright (C) 2015 EFKON AG http://www.efkon.com
 *
 * Developed by ITEG IT-Engineers GmbH http://www.iteg.at
 *
 ************************************************************/

#ifndef FANCYLOGD_LOGCLIENT_H
#define FANCYLOGD_LOGCLIENT_H

#include <fancyobject.h>
#include <fancylogdClient.h>

namespace org {
namespace clazzes {
namespace fancylogd {
namespace data {
class LogMessage;
}}}}

namespace Fancy { namespace LogD {

class FANCYLOGD_CLIENT_API LogClient : virtual public Fancy::Object {

public:
	typedef Fancy::ObjPtr<LogClient> ptr_t;

	/**
     Return a singleton instance and aumatically register it with
     fancylog_init_backend().
     A background thread, which feeds the pipe
     "/run/fancylogd.sock" (UNIX) or "\\\\.\\pipe\\fancylogd" (WIN32)
     is also started.

     If the returned instance is destroyed, the background thread is
     stopped and no more logging takes places.

     @c ident is sent to the server in a head request in order to
     configure an identification of the logged messages and provide
     a hint to the server for dispatching received messages to
     multiple log file, if configured to do so.
	 */
	static ptr_t init(const std::string &ident, int maxBufferSize = 1000);

	static ptr_t init(
			const std::string & tcpAddressPort,
			const std::string &ident,
			int maxBufferSize = 1000);

	static void closeLog();

	// a new connections starts by sending a string containing the ident
	// from then only LogMessages are sent

	// later.
	// Fancy::Sock::Address or Fancy::Sock::Device?
	//static ptr_t init(const std::string &ident, const std::string &daemonAdress)

	// later: commit policy?
	// max. queue size, i.e. 1000. can be hit if fancylogd goes away or if an endless loop hits us
	// what to do with new messages after queue has hit maximum size? local failback? discard?
	// client thread can be stuck in blocking network (re)connect
	// don't create endless context switches!

	// reconnect is a must, not part of a policy option

	// fallback logging? i.e. if queue runs full
	// https://msdn.microsoft.com/en-us/library/system.diagnostics.eventlog_methods(v=vs.110).aspx

	virtual void pushMessage(const
	Fancy::ObjPtr<org::clazzes::fancylogd::data::LogMessage> &msg) = 0;

	//virtual void pushMessages(
	//		const Fancy::ObjPtr<Fancy::List> &msg);

	/**
     Push message to the singleton initialized by @c init()
	 */
	static void pushMessage_s(const
	Fancy::ObjPtr<org::clazzes::fancylogd::data::LogMessage> &msg);

	virtual void requestInterruption() = 0;

};

}} // end namespaces

#endif
